﻿#!/usr/bin/env pwsh

#cd %~dp0
cls

$InterfaceDescription = "TAP-Windows Adapter V9"
$InterfaceNewName = "tun"

function CheckNodeJS {
    try {
        $NODE = node -v
        $NPM = npm -v
        Write-Host "[node] $NODE ✓" -ForegroundColor Green
        Write-Host "[npm] $NPM ✓" -ForegroundColor Green
    }
    catch [System.Management.Automation.CommandNotFoundException] {
        Write-Host "[node] NodeJS not install." -ForegroundColor DarkRed
        Write-Host "[node] Start installing NodeJS LTS..."
        # winget install OpenJS.NodeJS.LTS
        $NodeInstaller = ".\archives\node-v20.11.1-x64.msi"
        Start-Process -FilePath $NodeInstaller -Wait
        Write-Host "[node] NodeJS install successful." -ForegroundColor Green

        # refresh the env
        $env:Path = [System.Environment]::GetEnvironmentVariable("Path","Machine") + ";" + [System.Environment]::GetEnvironmentVariable("Path","User")
        CheckNodeJS
    }
}

function CheckZeus {
    try {
        $ZEUS = zeus -v
        Write-Host "[zeus] $ZEUS ✓" -ForegroundColor Green
    }
    catch [System.Management.Automation.CommandNotFoundException] {
        Write-Host "[zeus] @zeppos/zeus-cli not install." -ForegroundColor DarkRed
        Write-Host "[zeus] Start installing @zeppos/zeus-cli..."
        npm i @zeppos/zeus-cli -g
        Write-Host "[zeus] @zeppos/zeus-cli install successful." -ForegroundColor Green

        # refresh the env
        # $env:Path = [System.Environment]::GetEnvironmentVariable("Path","Machine") + ";" + [System.Environment]::GetEnvironmentVariable("Path","User")
        CheckZeus
    }
}

function RunTapInstaller {
    $TapInstaller = ".\archives\tap-windows-9.21.2.exe"
    Start-Process -FilePath $TapInstaller -Wait
}

function SetupTapInterface {
    $IP = "192.168.166.1"
    # This means subnet mask = 255.255.255.0
    $MaskBits = 24
    $IPType = "IPv4"

    # setup the tap network interface
    $adapter = Get-NetAdapter -InterfaceDescription $InterfaceDescription
    Write-Host "Find Adapter $($adapter.Name) $($adapter.InterfaceDescription)"
    Write-Host "Rename TAP Adapter:"
    Rename-NetAdapter -Name $adapter.Name -NewName $InterfaceNewName
    Write-Host "Network adapter name changed: $($InterfaceNewName)"

    # Get the adapter after name changed
    $adapter = Get-NetAdapter -InterfaceDescription $InterfaceDescription
    if (($adapter | Get-NetIPConfiguration).IPv4Address.IPAddress) {
        $adapter | Remove-NetIPAddress -AddressFamily $IPType -Confirm:$false
    }

    Write-Host "Set New IpAddress for Simulator: "
    # Disable DHCP
    Set-ItemProperty -Path ("HKLM:\SYSTEM\CurrentControlSet\services\Tcpip\Parameters\Interfaces\{0}" -f $adapter.DeviceID) -Name EnableDHCP -Value 0
    $adapter | New-NetIPAddress `
        -AddressFamily $IPType `
        -IPAddress $IP `
        -PrefixLength $MaskBits

    # Disable IPv6
    Disable-NetAdapterBinding -Name $adapter.Name -ComponentID ms_tcpip6
}

function CheckTapInterface {
    $adapter = Get-NetAdapter -InterfaceDescription $InterfaceDescription
    if (!$?) {
        Write-Host "[taptun] Can not find TAP Adapter.\n" -ForegroundColor DarkRed
        Write-Host "[taptun] Start install taptun driver..." -ForegroundColor DarkCyan
        RunTapInstaller -and
        CheckTapInterface
        break
    } else {
        if ($adapter.Name -ne $InterfaceNewName) {
            SetupTapInterface
        } else {
            Write-Host "[taptun] Network Interface Name: $($adapter.Name), IPAddress: $(($adapter | Get-NetIPConfiguration).IPv4Address.IPAddress[0]) ✓" -ForegroundColor Green
        }
    }
}

CheckNodeJS -and
CheckZeus -and
CheckTapInterface
